#include <stdlib.h>
#include <gnome.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/parser.h>

#include "rubrica.h"
#include "modulo.h"
#include "main.h"
#include "file.h"

VistaDati vista;

static gchar *file;
static gchar *path;
static gchar *nomefile;

void rubrica_init_archivio(void)
{
  gint n;    
  gchar *str;
  InfoRubrica i;
  GtkCList *schede;
  GtkCList *dati;
  
  /* libero la memoria occupata da rubrica */
  rubrica_svuota_memoria();
  
  /* azzero il numero totale di indirizzi */
  n = 0;
  rubrica_vista_set_indirizzi(n);
  
  /* cancello il nome del file visualizzato */
  str = NULL;
  rubrica_vista_set_label(str);
  
  /* imposto i valori booleani sullo stato della rubrica */
  rubrica_modificata(FALSE);

  /* inizializzo i valori della clist contenente 
     l'elenco delle schede 
  */
  i.riga = 0;
  i.colonna = 0;
  i.selezionato = FALSE;
  rubrica_vista_set_info(i, VISTA_SCHEDE);
  rubrica_vista_set_info(i, VISTA_DATI);
  
  /* pulisco le clist
     - schede
     - dati
  */
  schede = rubrica_vista_get_schede();
  dati   = rubrica_vista_get_dati();
  
  gtk_clist_clear(GTK_CLIST(schede));
  gtk_clist_clear(GTK_CLIST(dati));  
  
  /* inizializzo i ptr ai nomi del file */
  if (file)
    g_free(file);
  if (path)
    g_free(path);
  if(nomefile)
    g_free(nomefile);
  
  file = NULL;
  path = NULL;
  nomefile = NULL;
}

void 
rubrica_crea_nuovo_archivio(void)
{
  gint scelta;
  gchar *mes;
  
  if (rubrica_is_modificata())
    {
      mes = g_strdup(_("Attenzione. La rubrica  stata modificata.\nI cambiamenti\
 non salvati saranno persi.\n Continuo?")); 
      
      scelta = confermare_uscita(mes);
      g_free(mes);
      
      switch(scelta)
	{
	case 0:
	  rubrica_init_archivio();
	  break;
	  
	case 1:
	default:
	  break;
	}  
    }
  else 
    rubrica_init_archivio();
}



void 
rubrica_salva_file(GtkWidget *w, gpointer data)
{
  GtkWidget *app;

  app = rubrica_get_app();
  /* se non c' un nome file di default
     allora bisogna richiamare salva_con_nome
  */
  if (!nomefile)
    rubrica_salva_file_come(NULL, NULL);
  /* altrimenti provo a scrivere il file */
  else 
    if (!write_xml_file(nomefile)) 
	gnome_app_error(GNOME_APP(app), _("Impossibile salvare il file"));        
}

void 
rubrica_salva_file_come(GtkWidget *w, gpointer data)
{
  GtkWidget *app;
  GtkFileSelection *fs;
  
  app = rubrica_get_app();
  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Salva come")));
  
  /* se ho gi un nome di file lo mostro nell'entry*/
  if (nomefile)
       gtk_file_selection_set_filename(fs, nomefile);
  
  /* connetto i bottoni ok e cancella */
  gtk_signal_connect(GTK_OBJECT(fs->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(rubrica_scrivi_file), fs);
  
  /* con il bottone cancella distruggo direttamente il widget */
  gtk_signal_connect_object(GTK_OBJECT(fs->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(fs));

  gtk_window_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));
  
  gtk_widget_show(GTK_WIDGET(fs));
}


void
rubrica_scrivi_file(GtkWidget *w, GtkFileSelection *fs)
{
  gchar *fname;

  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));

  fname = gtk_file_selection_get_filename(fs);

  /* se non  stato selezionato nessun file
     o non  possibile aprire il file segnalare l'errore */
  if ( !fname || !write_xml_file(fname)) {
    GtkWidget *dlg;

    /* il dialogo di errore ha il widget fs come genitore */
    dlg = gnome_error_dialog_parented(_("Impossibile salvare il file"), 
				      GTK_WINDOW(fs));
    gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);   
  }
  else /* il file  stato salvato */
    {
      /* imposto l'etichetta del nome file */
      if (rubrica_get_mostra_path())
	rubrica_vista_set_label(path);
      else
	rubrica_vista_set_label(nomefile);
      
      gtk_widget_destroy(GTK_WIDGET(fs));
    }
}

gboolean
write_xml_file(gchar *fname)
{
  /* puntatori al documento xml */
  xmlDocPtr doc;
  GList *l;
  GList *rub;
    
  rub = rubrica_get_rubrica();

  g_return_val_if_fail(fname != NULL, FALSE);
  g_return_val_if_fail(rub != NULL, FALSE);
  
  /* creazione di un nuovo documento xml */
  doc = xmlNewDoc("1.0");
  
  /* creo un nuovo nodo radice rubrica */
  doc->root = xmlNewDocNode(doc, NULL, "Rubrica", NULL);

  /* attraverso la rubrica e scrivo i dati */
  for (l = rub; l != NULL; l = l->next) {
    RubricaItem *item = l->data;

    rubrica_write_elemento_to_xml(doc->root, item);
  }

  if (xmlSaveFile(fname, doc) == -1) {
    xmlFreeDoc(doc);
    return FALSE;
  }
  if (file)
    g_free(file);
  if (path)
    g_free(path);
  if(nomefile)
    g_free(nomefile);
  
  file = g_strdup(fname);
  path = g_strdup(g_dirname(fname));
  nomefile = g_strdup(g_basename(fname));

  xmlFreeDoc(doc);    

  return TRUE;
}

void 
rubrica_write_elemento_to_xml(xmlNodePtr xmlnode, RubricaItem *item)
{
  GList *alias;

  xmlNodePtr newxml;

  xmlNodePtr boolxml;
  xmlNodePtr schedaxml;
  xmlNodePtr datixml;
  xmlNodePtr webxml;
  xmlNodePtr mailxml;
  xmlNodePtr telxml;
  xmlNodePtr notexml;

  gint righe;
  gchar *str;

  g_return_if_fail(xmlnode != NULL);
  g_return_if_fail(item != NULL);

  /*
    newxml  figlio di xmlnode (il primo xmlnode  doc->root)
    boolxml, schedaxml, datixml, ..., sono figli di newxml
  */
    
  newxml = xmlNewChild(xmlnode, NULL, "Indirizzo", NULL);
  
  boolxml = xmlNewChild(newxml, NULL, "Editabilit", NULL);
  if (item->editabile)
    xmlSetProp(boolxml, "editabile", "TRUE");
  else
    xmlSetProp(boolxml, "editabile", "FALSE");
  if (item->cancellabile)
    xmlSetProp(boolxml, "cancellabile", "TRUE");
  else
    xmlSetProp(boolxml, "cancellabile", "FALSE");
  
  schedaxml = xmlNewChild(newxml, NULL, "Scheda", NULL);
  xmlSetProp(schedaxml, "scheda", item->scheda);
  
  datixml = xmlNewChild(newxml, NULL, "Dati", NULL);
  xmlSetProp(datixml, "nome", item->nome);
  xmlSetProp(datixml, "cognome", item->cognome);
  xmlSetProp(datixml, "professione", item->professione);
  xmlSetProp(datixml, "via", item->via);
  xmlSetProp(datixml, "numero", item->numero);
  xmlSetProp(datixml, "cap", item->cap);
  xmlSetProp(datixml, "citt", item->citta);
  xmlSetProp(datixml, "provincia", item->provincia);
  xmlSetProp(datixml, "stato", item->stato);

  webxml = xmlNewChild(newxml, NULL, "Web", NULL);
  righe = item->nweb;
  alias = item->web_list;
  str = g_strdup_printf("%d",righe);
  xmlSetProp(webxml, "nweb", str);
  while(righe)
    {
      gchar *str;
      
      str = g_strdup_printf("web%d",righe);
      xmlSetProp(webxml, str,alias->data); 
      
      alias = alias->next;
      righe--;
    }
  
  mailxml = xmlNewChild(newxml, NULL, "Email", NULL);
  righe = item->nmail;
  alias = item->mail_list;
  str = g_strdup_printf("%d",righe);
  xmlSetProp(mailxml, "nmail", str);
  while (righe)
    {
      gchar *str;

      str = g_strdup_printf("mail%d", righe);
      xmlSetProp(mailxml, str, alias->data);
      alias = alias->next;
      righe--;
    }

  telxml = xmlNewChild(newxml, NULL, "Telefono", NULL);
  righe = item->ntel;
  alias = item->tel_list;
  str = g_strdup_printf("%d",righe);
  xmlSetProp(telxml, "ntel", str);
  while(righe)
    {
      gchar *num;
      gchar *tipo;
      TelNum *tt = (TelNum*) alias->data;

      num  = g_strdup_printf("telefono%d", righe);
      tipo = g_strdup_printf("tipo%d", righe);

      xmlSetProp(telxml, num, tt->numero_telefono);
      xmlSetProp(telxml, tipo, tt->tipo);
      alias = alias->next;
      righe--;
    }
  
  notexml = xmlNewChild(newxml, NULL, "Note", NULL);
  xmlSetProp(notexml, "note", item->note);
}


void 
rubrica_carica_file(void)
{
  GtkFileSelection *fs;
  GtkWidget *app;

  app = rubrica_get_app();

  fs = GTK_FILE_SELECTION(gtk_file_selection_new(_("Apri file...")));
  
  /* connetto i bottoni ok e cancel */
  gtk_signal_connect(GTK_OBJECT(fs->ok_button), "clicked",
		     GTK_SIGNAL_FUNC(rubrica_apri_file), fs);
  gtk_signal_connect_object(GTK_OBJECT(fs->cancel_button), "clicked",
			    GTK_SIGNAL_FUNC(gtk_widget_destroy), 
			    GTK_OBJECT(fs));

  /* visualizzo la finestra in posizione centrale */
  gtk_window_position(GTK_WINDOW(fs), GTK_WIN_POS_CENTER);

  /* il genitore della finestra fileselection  app */
  gtk_window_set_transient_for(GTK_WINDOW(fs), GTK_WINDOW(app));

  gtk_widget_show(GTK_WIDGET(fs));
}

void 
rubrica_apri_file(GtkWidget *w, GtkFileSelection *fs)
{
  gchar *fname;

  g_return_if_fail(GTK_IS_FILE_SELECTION(fs));

  fname = gtk_file_selection_get_filename(fs);

  if (!fname || !read_xml_file(fname))
    {
      GtkWidget *dlg;
      
      dlg = gnome_error_dialog_parented(_("Impossibile aprire il file"), 
					GTK_WINDOW(fs));
      gtk_window_set_modal(GTK_WINDOW(dlg), TRUE);
    }
  else
    {
      if (rubrica_get_mostra_path())
	rubrica_vista_set_label(path);
      else
	rubrica_vista_set_label(nomefile);

      gtk_widget_destroy(GTK_WIDGET(fs));
    }
}


gboolean 
read_xml_file(gchar *fname)
{
  xmlDocPtr doc;

  g_return_val_if_fail(fname != NULL, FALSE);
  
  /* se il file non esiste */
  if (!g_file_exists(fname))
    return FALSE;

  doc = xmlParseFile(fname);

  if (!doc)
    return FALSE;

  if (!doc->root || !doc->root->name || 
      g_strcasecmp(doc->root->name, "Rubrica") != 0)
    {
      xmlFreeDoc(doc);
      return FALSE;
    }
  parse_doc(doc);

  xmlFreeDoc(doc);

  if (file)
    g_free(file);
  if(path)
    g_free(path);
  if(nomefile)
    g_free(nomefile);
  
  file = g_strdup(fname);
  path = g_strdup(g_dirname(fname));
  nomefile = g_strdup(g_basename(fname));
  
  return TRUE;  
}


void 
parse_doc(xmlDocPtr doc)
{  
  xmlNodePtr nodo;

  /* inizializzo le strutture necessarie per 
     caricare una nuova lista
  */
  rubrica_crea_nuovo_archivio();

  /* prendo il nodo Indirizzo dal file e lo inserisco nella 
     (nuova) rubrica 
     il primo nodo  doc->root->childs == xmlnew (salvato)
  */
  for(nodo = doc->root->childs; nodo != NULL; nodo=nodo->next)
    rubrica_load_elemento_from_xml(nodo, NULL);  
}

void
rubrica_load_elemento_from_xml(xmlNodePtr xmlnode, GList *l)
{
  gint n;
  RubricaItem *r;  
  TelNum *t;
  GList *web, *mail, *tel;

  /* i nodi xmlnode dovrebbero essere nodi xmlnew
     - devo estrarre i figli di xmlnew == xmldoc che sono:
       boolxml, schedaxml, ...
  */
  xmlNodePtr figlio;

  if (!xmlnode->name ||  /* se il nodo non ha nome */
      g_strcasecmp(xmlnode->name, "Indirizzo") != 0) /* il nome non  Indirizzo */
    return;
  
  r = g_malloc(sizeof(RubricaItem));

  /* valori booleani */
  figlio = xmlnode->childs; 
  if (!figlio->name || g_strcasecmp(figlio->name, "Editabilit") != 0)
    {
      /* non trovo i valori, imposto il default */
      r->editabile = TRUE;
      r->cancellabile = TRUE;
    }
  else 
    {
      gchar *buffer;
      
      buffer = xmlGetProp(figlio, "editabile");
      if (g_strcasecmp(buffer, "TRUE") == 0)
	r->editabile = TRUE;
      else 
	r->editabile = FALSE;
      
      buffer = xmlGetProp(figlio, "cancellabile");
      if (g_strcasecmp(buffer, "TRUE") == 0)
	r->cancellabile = TRUE;
      else 
	r->cancellabile = FALSE;      
    }

  figlio = figlio->next; // schedaxml ??
  if (!figlio->name || g_strcasecmp(figlio->name, "Scheda") != 0)
    r->scheda = g_strdup("");
  else 
    r->scheda = g_strdup(xmlGetProp(figlio, "scheda"));
  
  // dati personali
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Dati") != 0)
    r->nome = r->cognome 
      = r->professione = r->via 
      = r->numero = r->cap 
      = r->citta = r->provincia 
      = r->stato = g_strdup("");
  else 
    {
      r->nome = g_strdup(xmlGetProp(figlio, "nome"));
      r->cognome = g_strdup(xmlGetProp(figlio, "cognome"));
      r->professione = g_strdup(xmlGetProp(figlio, "professione"));
      r->via = g_strdup(xmlGetProp(figlio, "via"));
      r->numero = g_strdup(xmlGetProp(figlio, "numero"));
      r->cap = g_strdup(xmlGetProp(figlio, "cap"));
      r->citta = g_strdup(xmlGetProp(figlio, "citt"));
      r->provincia = g_strdup(xmlGetProp(figlio, "provincia"));
      r->stato = g_strdup(xmlGetProp(figlio, "stato"));
    }

  //web
  web = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Web") != 0)
    {
      r->web_list = NULL;
      r->nweb = 0;
    }
  else
    {
      gchar *buffer, *str, *tmp;
            
      tmp = xmlGetProp(figlio, "nweb");
      n = r->nweb = atoi(tmp);      
      while(n)
	{
	  str = g_strdup_printf("web%d",n);
	  buffer = g_strdup(xmlGetProp(figlio,str));
	  
	  web = g_list_append(web, buffer);
	  n--;
	}
      r->web_list = web;
    }
  
  // mail
  mail = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Email") != 0)
    {
      r->mail_list = NULL;
      r->nmail = 0;
    }
  else
    {
      gchar *buffer, *str, *tmp;
      
      tmp= xmlGetProp(figlio, "nmail");      
      n = r->nmail = atoi(tmp);   
      while(n)
	{
	  str = g_strdup_printf("mail%d",n);
	  buffer = g_strdup(xmlGetProp(figlio,str));
	  
	  mail = g_list_append(mail, buffer);
	  n--;
	}
      r->mail_list = mail;
    }  
    
  // tel
  tel = NULL;
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Telefono") != 0)
    {
      r->tel_list = NULL;
      r->ntel = 0;
    }
  else
    {
      gchar *num, *tipo, *tmp;
      
      tmp = xmlGetProp(figlio, "ntel");
      n = r->ntel = atoi(tmp);   
      while(n)
	{
	  t = g_malloc(sizeof(TelNum));
	  
	  num  = g_strdup_printf("telefono%d",n);
	  tipo = g_strdup_printf("tipo%d",n);

	  t->numero_telefono = g_strdup(xmlGetProp(figlio,num));
	  t->tipo = g_strdup(xmlGetProp(figlio,tipo));
	  
	  tel = g_list_append(tel, t);
	  n--;
	}
      r->tel_list = tel;
    }
  
  // note
  figlio = figlio->next;
  if (!figlio->name || g_strcasecmp(figlio->name, "Note") != 0)
    r->note = g_strdup("");
  else 
    r->note = g_strdup(xmlGetProp(figlio,"note"));

  rubrica_inserisci_elemento(r);
  rubrica_vista_inc_indirizzi();
  rubrica_inserisci_scheda(r);
}





